<?php

App::uses('CakeEventListener', 'Event');

class BulkDiscountBasketListener implements CakeEventListener {

	public function implementedEvents() {
		return array(
			'EvBasket.Component.Basket.itemAdd' => array(
				'callable' => 'bulkDiscount',
				'priority' => 1
			),
			'EvBasket.Component.Basket.itemUpdated' => array(
				'callable' => 'bulkDiscount',
				'priority' => 1
			),
			'EvBasket.Component.Basket.itemDeleted' => array(
				'callable' => 'bulkDiscount',
				'priority' => 1
			)
		);
	}

	/**
	 * rebuild the bulk discount in the basket
	 * $Event->subject() = BasketManager Component
	 */
	public function bulkDiscount(CakeEvent $Event) {
		if (! CakePlugin::loaded('EvBasket')) {
			return false;
		}

		$basket = $Event->subject()->getBasket(true);

		// get the product ids from the basket
		$productIds = Hash::extract($basket, 'BasketItem.{s}.Variant.product_id');

		// get any bulk discountg data
		$BulkDiscount = EvClassRegistry::init('EvBulkDiscount.BulkDiscount');
		$discounts = $BulkDiscount->find(
			'all',
			array(
				'conditions' => array(
					'BulkDiscount.model' => 'EvShop.Product',
					'BulkDiscount.model_id' => $productIds
				),
				'order' => 'BulkDiscount.model_id ASC, BulkDiscount.quantity ASC'
			)
		);

		$discounts = Hash::combine(
			$discounts,
			'{n}.BulkDiscount.quantity',
			'{n}',
			'{n}.BulkDiscount.model_id'
		);

		if (! empty($basket['BasketItem'])) {
			$basketQuantities = Hash::combine(
				$basket['BasketItem'],
				array(
					'%s.%s',
					'{s}.model',
					'{s}.model_id'
				),
				'{s}.quantity',
				'{s}.Variant.product_id'
			);

			$originalUnitPrices = array();
			$toUpdate = array();

			foreach ($basketQuantities as $productId => $variants) {
				$totalQuantity = array_sum($variants);

				$quantities = array_keys($discounts[$productId]);
				sort($quantities);

				$matched = 0;
				foreach ($quantities as $quantity) {
					if ($totalQuantity >= $quantity) {
						$matched = $quantity;
					}
				}

				if (! empty($discounts[$productId][$matched])) {
					$discount = $discounts[$productId][$matched];

					foreach ($variants as $basketId => $quantity) {
						$price = $basket['BasketItem'][$basketId]['Variant']['price'];

						if (isset($basket['BasketItem'][$basketId]['Variant']['sale_price']) && $basket['BasketItem'][$basketId]['Variant']['sale_price'] > 0) {
							$price = $basket['BasketItem'][$basketId]['Variant']['sale_price'];
						}

						$originalUnitPrices[$basketId] = $price;
						$perc = ($discount['BulkDiscount']['amount'] / 100);
						$unitPrice = number_format($price - ($price * $perc), 2);

						$toUpdate[] = array(
							'model' => $basket['BasketItem'][$basketId]['model'],
							'model_id' => $basket['BasketItem'][$basketId]['model_id'],
							'quantity' => $quantity,
							'unitPrice' => $unitPrice,
							'taxRate' => $basket['BasketItem'][$basketId]['tax_rate']
						);
					}
				} else {
					foreach ($variants as $basketId => $quantity) {
						$price = $basket['BasketItem'][$basketId]['Variant']['price'];

						if (isset($basket['BasketItem'][$basketId]['Variant']['sale_price']) && $basket['BasketItem'][$basketId]['Variant']['sale_price'] > 0) {
							$price = $basket['BasketItem'][$basketId]['Variant']['sale_price'];
						}

						$toUpdate[] = array(
							'model' => $basket['BasketItem'][$basketId]['model'],
							'model_id' => $basket['BasketItem'][$basketId]['model_id'],
							'quantity' => $quantity,
							'unitPrice' => $price,
							'taxRate' => $basket['BasketItem'][$basketId]['tax_rate']
						);
					}
				}
			}

			if (! empty($toUpdate)) {
				$Event->subject()->updateItem($toUpdate);
			}

			$Event->subject()->_controller->set('originalUnitPrices', $originalUnitPrices);
			$Event->subject()->_controller->BasketCookie->write(
				'originalUnitPrices',
				$originalUnitPrices
			);
		}
	}

}
